# Implements a "smart" command line widget including history, etc.

from PyQt4 import QtGui, QtCore

class CmdLine(QtGui.QLineEdit):
    def __init__(self, parent):
        QtGui.QLineEdit.__init__(self, parent)

        self.older_history = []
        self.newer_history = []
        self.current_hist_item = None

    def keyPressEvent(self, event):
        key = event.key()
        mod = event.modifiers()
        
        if key == QtCore.Qt.Key_Up and mod == QtCore.Qt.ControlModifier:
            if len(self.older_history) == 0:
                return

            if self.current_hist_item:
                self.newer_history.append(self.current_hist_item)
            else:
                self.newer_history.append(self.displayText())

            self.current_hist_item = self.older_history.pop()
            self.setText(self.current_hist_item)
            return

        if key == QtCore.Qt.Key_Down and mod == QtCore.Qt.ControlModifier:
            if len(self.newer_history) == 0:
                return

            self.older_history.append(self.current_hist_item)
            self.current_hist_item = self.newer_history.pop()
            self.setText(self.current_hist_item)

            if len(self.newer_history) == 0:
                self.current_hist_item = None
            return

        if key == QtCore.Qt.Key_Return:
            self.older_history += self.newer_history
            self.newer_history = []
            self.current_hist_item = None

            cmd = self.displayText()
            if str(cmd) != '':
                self.older_history.append(self.displayText())
            self.emit(QtCore.SIGNAL('returnPressed()'))
            return

        QtGui.QLineEdit.keyPressEvent(self, event)

        
