import os, sys, tempfile

from PyQt4 import QtCore, QtGui

import pmpu_main, about, archive, clone, favourites, new_fav, prefs, tag, \
    init_repo, update, email, annotate

import scm

class About(QtGui.QDialog):
    def __init__(self):
        QtGui.QWidget.__init__(self)

        self.aboutbox = about.Ui_Dialog()
        self.aboutbox.setupUi(self)

class Clone(QtGui.QDialog):
    def __init__(self, settings, backends):
        QtGui.QWidget.__init__(self)
        
        self.cloneDialog = clone.Ui_Dialog()
        self.cloneDialog.setupUi(self)
        self.connect(self.cloneDialog.openLocal,
                     QtCore.SIGNAL('clicked()'), self.openLocal)
        self.connect(self.cloneDialog.openFavourite,
                     QtCore.SIGNAL('clicked()'), self.openFavourite)
        self.connect(self.cloneDialog.chooseLocalDest,
                     QtCore.SIGNAL('clicked()'), self.chooseLocalDest)
        self.settings = settings

        for b in backends:
            if b.features[scm.SCM_CLONE]:
                self.cloneDialog.scmType.addItem(b.name)

    def openLocal(self):
        path = QtGui.QFileDialog.getExistingDirectory(self)
        if path == '':
            return
        self.cloneDialog.sourcePath.setText(path)

    def openFavourite(self):
        f = Favourites(self.settings)
        if f.exec_() == 0:
            return
        i = f.selectedLoc()
        if i:
            self.cloneDialog.sourcePath.setText(i)

    def chooseLocalDest(self):
        path = QtGui.QFileDialog.getSaveFileName(self)
        if path == '':
            return
        self.cloneDialog.destPath.setText(path)

    def scmName(self):
        return str( self.cloneDialog.scmType.currentText() )

class Favourites(QtGui.QDialog):
    def __init__(self, settings):
        QtGui.QWidget.__init__(self)
        self.favBox = favourites.Ui_Dialog()
        self.favBox.setupUi(self)
        self.connect(self.favBox.addBookmarkButton,
                     QtCore.SIGNAL('clicked()'),
                     self.newFav)
        self.connect(self.favBox.deleteButton,
                     QtCore.SIGNAL('clicked()'),
                     self.deleteFav)
        self.connect(self.favBox.editButton,
                     QtCore.SIGNAL('clicked()'),
                     self.editFav)
        self.settings = settings
        self.settings.beginGroup('favourites')
        for desc in self.settings.allKeys():
            # for each item, keyed by description
            loc = settings.value(desc, QtCore.QVariant('')).toString()
            self.favBox.favouritesList.addItem(desc)
        self.settings.endGroup()

    def newFav(self):
        nf = NewFav()
        if nf.exec_() == 0:
            return

        favLocation = nf.newfav.newFavName.displayText()
        favDescription = nf.newfav.newFavDescription.displayText()

        self.favBox.favouritesList.addItem(favDescription)
        self.settings.setValue('favourites/' +
                               favDescription, QtCore.QVariant(favLocation))

    def selectedLoc(self):
        k = self.selectedKey()
        if not k:
            return k

        loc = self.settings.value('favourites/' + k,
                                  QtCore.QVariant('')).toString()

        return loc

    def selectedKey(self):
        return self.favBox.favouritesList.currentItem().text()

    def deleteFav(self):
        self.settings.remove('favourites/' + self.selectedKey())
        self.favBox.favouritesList.takeItem(
            self.favBox.favouritesList.currentRow())

    def editFav(self):
        nf = NewFav(self.selectedLoc(), self.selectedKey())
        if nf.exec_() == 0:
            return
        
        favLocation = nf.newfav.newFavName.displayText()
        favDescription = nf.newfav.newFavDescription.displayText()

        self.settings.remove('favourites/' + self.selectedKey())

        current = self.favBox.favouritesList.currentItem()
        current.setText(favDescription)

        self.settings.setValue('favourites/' +
                               favDescription, QtCore.QVariant(favLocation))
        

class NewFav(QtGui.QDialog):
    def __init__(self, loc=QtCore.QString(''), desc=QtCore.QString('')):
        QtGui.QWidget.__init__(self)
        self.newfav = new_fav.Ui_Dialog()
        self.newfav.setupUi(self)
        self.newfav.newFavName.setText(loc)
        self.newfav.newFavDescription.setText(desc)
        

class Prefs(QtGui.QDialog):
    def __init__(self, scm, settings):
        QtGui.QWidget.__init__(self)
        self.prefsBox = prefs.Ui_Dialog()
        self.prefsBox.setupUi(self)

        emailProgPath = settings.value('preferences/email',
                                       QtCore.QVariant('')).toString()
        self.prefsBox.emailProg.setText(emailProgPath)

        histview = settings.value('preferences/%s/histview' % scm.name,
                                  QtCore.QVariant('')).toString()
        self.prefsBox.histViewPath.setText(histview)
        ct = settings.value('preferences/%s/ctpath' % scm.name,
                            QtCore.QVariant('')).toString()
        self.prefsBox.commitToolPath.setText(ct)

class Archive(QtGui.QDialog):
    def __init__(self, scm):
        QtGui.QWidget.__init__(self)
        self.archiveBox = archive.Ui_Dialog()
        self.archiveBox.setupUi(self)

        for k in scm.archive_formats.keys():
            if scm.archive_formats[k]:
                self.archiveBox.archiveTypes.addItem(k)

        self.connect(self.archiveBox.chooseFileButton,
                     QtCore.SIGNAL('clicked()'),
                     self.chooseFile)

    def getTypeInfo(self):
        type = self.archiveBox.archiveTypes.currentText()

        if type == scm.ARCHIVE_FILES:
            return '', ['']
        elif type == scm.ARCHIVE_TAR:
            return 'Uncompressed Tar Files (*.tar)', ['.tar']
        elif type == scm.ARCHIVE_TAR_BZ2:
            return 'Bzip2ed Tar Files (*.tar.bz2 *.tbz2)', ['.tar.bz2', '.tbz2']
        elif type == scm.ARCHIVE_TAR_GZ:
            return 'Gzipped Tar Files (*.tar.gz *.tgz)', ['.tar.gz', '.tgz']
        elif type == scm.ARCHIVE_ZIP_UNCOMPRESSED:
            return 'Zip Files (*.zip)', ['.zip']
        elif type == scm.ARCHIVE_ZIP_COMPRESSED:
            return 'Zip Files (*.zip)', ['.zip']

    def chooseFile(self):
        filter, valid_endings = self.getTypeInfo()
        archivePath = QtGui.QFileDialog.getSaveFileName(self,
                                                        'Save Archive',
                                                        '.',
                                                        filter)

        add_extension = valid_endings[0]
        for e in valid_endings:
            if archivePath.endsWith(e):
                add_extension = ''
                
        if archivePath.length() == 0:
            add_extension = ''

        archivePath.append(QtCore.QString(add_extension))

        self.archiveBox.archiveFile.setText(archivePath)

class Tag(QtGui.QDialog):
    def __init__(self):
        QtGui.QDialog.__init__(self)

        self.tagBox = tag.Ui_Dialog()
        self.tagBox.setupUi(self)

    def name(self):
        return str( self.tagBox.name.displayText() )

class InitRepo(QtGui.QDialog):
    def __init__(self, backends):
        QtGui.QDialog.__init__(self)

        self.initRepoBox = init_repo.Ui_Dialog()
        self.initRepoBox.setupUi(self)

        self.connect(self.initRepoBox.chooseExisting,
                     QtCore.SIGNAL('clicked()'),
                     self.chooseExisting)
        self.connect(self.initRepoBox.chooseNew,
                     QtCore.SIGNAL('clicked()'),
                     self.chooseNew)
        
        for b in backends:
            if b.features[scm.SCM_INIT]:
                self.initRepoBox.scmType.addItem(b.name)

    def chooseExisting(self):
        path = QtGui.QFileDialog.getExistingDirectory(self)
        if path != '':
            self.initRepoBox.repoPath.setText(path)

    def chooseNew(self):
        path = QtGui.QFileDialog.getSaveFileName(self)
        if path != '':
            self.initRepoBox.repoPath.setText(path)

    def repo(self):
        return str( self.initRepoBox.repoPath.displayText() )

    def scmName(self):
        return str( self.initRepoBox.scmType.currentText() )

class Update(QtGui.QDialog):
    def __init__(self, my_scm):
        QtGui.QDialog.__init__(self)
        self.ui = update.Ui_Dialog()
        self.ui.setupUi(self)

        self.ui.updateToCurrent.setEnabled(my_scm.update_modes[scm.UPDATE_TO_TIP])
        self.ui.updateToRev.setEnabled(my_scm.update_modes[scm.UPDATE_TO_REV])
        self.ui.rev.setEnabled(my_scm.update_modes[scm.UPDATE_TO_REV])
        self.ui.updateToTime.setEnabled(my_scm.update_modes[scm.UPDATE_TO_TIME])
        self.ui.time.setEnabled(my_scm.update_modes[scm.UPDATE_TO_TIME])

        self.my_scm = my_scm

    def type(self):
        if self.ui.updateToCurrent.isChecked():
            return scm.UPDATE_TO_TIP
        elif self.ui.updateToRev.isChecked():
            return scm.UPDATE_TO_REV
        elif self.ui.updateToTime.isChecked():
            return scm.UPDATE_TO_TIME

    def rev(self):
        return str( self.ui.rev.displayText() )

    def time(self):
        return self.ui.time.dateTime().toString(self.my_scm.time_format)

class Email(QtGui.QDialog):
    def __init__(self):
        QtGui.QDialog.__init__(self)

        self.ui = email.Ui_Dialog()
        self.ui.setupUi(self)

    def bundleName(self):
        return self.ui.bundleName.displayText()

    def to(self):
        return self.ui.toAddress.displayText()

    def subject(self):
        return self.ui.subject.displayText()

    def message(self):
        return self.ui.message.toPlainText()

class Annotate(QtGui.QDialog):
    FILTER_ANY = 'any'
    FILTER_USER = 'user'
    FILTER_REVNO = 'revision no'
    FILTER_REVID = 'revision ID'
    FILTER_DATE = 'date'
    FILTER_FILE = 'file'
    FILTER_CODE = 'code'

    filterModes = [ FILTER_ANY, FILTER_USER, FILTER_REVNO, FILTER_REVID,
                    FILTER_DATE, FILTER_FILE, FILTER_CODE ]

    def __init__(self, annotated_lines):
        QtGui.QDialog.__init__(self)

        self.annotated_lines = annotated_lines

        self.ui = annotate.Ui_Dialog()
        self.ui.setupUi(self)

        self.connect(self.ui.showUser,
                     QtCore.SIGNAL('clicked()'),
                     self.render)
        self.connect(self.ui.showRevno,
                     QtCore.SIGNAL('clicked()'),
                     self.render)
        self.connect(self.ui.showRevid,
                     QtCore.SIGNAL('clicked()'),
                     self.render)
        self.connect(self.ui.showDate,
                     QtCore.SIGNAL('clicked()'),
                     self.render)
        self.connect(self.ui.showFile,
                     QtCore.SIGNAL('clicked()'),
                     self.render)
        self.connect(self.ui.showCode,
                     QtCore.SIGNAL('clicked()'),
                     self.render)
        self.connect(self.ui.filterMode,
                     QtCore.SIGNAL('currentIndexChanged(int)'),
                     self.render)
        self.connect(self.ui.filterText,
                     QtCore.SIGNAL('textChanged(const QString &)'),
                     self.render)
        self.connect(self.ui.showUnmatched,
                     QtCore.SIGNAL('clicked()'),
                     self.render)

        self.ui.filterMode.addItems(self.filterModes)

        self.render()

    class escapedLine:
        def __init__(self, annLine):
            self.annLine = annLine

        def __getitem__(self, item):
            return QtCore.Qt.escape(self.annLine[item])
        
    def render(self, idx=0):
        print 'Rendering...'
        formatMatch = ''
        if self.ui.showUser.isChecked():
            formatMatch += '<b>%(user)s</b> '
        if self.ui.showRevno.isChecked():
            formatMatch += '%(revno)s '
        if self.ui.showRevid.isChecked():
            formatMatch += '<i>%(revid)s</i> '
        if self.ui.showDate.isChecked():
            formatMatch += '%(date)s '
        if self.ui.showFile.isChecked():
            formatMatch += '%(file)s '
        if self.ui.showCode.isChecked():
            formatMatch += ': %(text)s'
        formatMatch += '<br/>'

        if self.ui.showUnmatched.isChecked():
            formatMiss = '<font color=\'gray\'>' + formatMatch + '</font>'
        else:
            formatMiss = ''

        filterMode = self.filterMode()
        filterText = self.filterText()

        code = ''
        for ann_line in self.annotated_lines:
            format = formatMatch

            if filterMode == self.FILTER_USER:
                if ann_line.user.find(filterText) == -1:
                    format = formatMiss
            elif filterMode == self.FILTER_REVNO:
                if ann_line.revno.find(filterText) == -1:
                    format = formatMiss
            elif filterMode == self.FILTER_REVID:
                if ann_line.revid.find(filterText) == -1:
                    format = formatMiss
            elif filterMode == self.FILTER_DATE:
                if ann_line.date.find(filterText) == -1:
                    format = formatMiss
            elif filterMode == self.FILTER_FILE:
                if ann_line.file.find(filterText) == -1:
                    format = formatMiss
            elif filterMode == self.FILTER_CODE:
                if ann_line.text.find(filterText) == -1:
                    format = formatMiss

            if filterMode == self.FILTER_ANY:
                if ann_line.toText().find(filterText) == -1:
                    format = formatMiss

            code += format % self.escapedLine(ann_line)

        self.ui.annotateView.setHtml(code)

    def filterText(self):
        return self.ui.filterText.displayText()


    def filterMode(self):
        return str( self.ui.filterMode.currentText() )

class PushMePullYou(QtGui.QMainWindow):
    def __init__(self, qapp):
        QtGui.QWidget.__init__(self)
        
        self.qapp = qapp
        self.ui = pmpu_main.Ui_MainWindow()
        self.ui.setupUi(self)

        self.connect(self.ui.actionAbout,
                     QtCore.SIGNAL('triggered()'), self.showAbout)
        self.connect(self.ui.actionOpen_path,
                     QtCore.SIGNAL('triggered()'), self.openPath)
        self.connect(self.ui.paths,
                     QtCore.SIGNAL('currentIndexChanged(int)'),
                     self.updateDisplays)
        self.connect(self.ui.pullIn, QtCore.SIGNAL('clicked()'),
                     self.pull)
        self.connect(self.ui.pushOut, QtCore.SIGNAL('clicked()'),
                     self.push)
        # Should probably update the paths list here too...
        self.connect(self.ui.actionRefresh,
                     QtCore.SIGNAL('triggered()'), self.updateDisplays)
        self.connect(self.ui.actionClone,
                     QtCore.SIGNAL('triggered()'), self.clone)
        self.connect(self.ui.currentRepo,
                     QtCore.SIGNAL('returnPressed()'),
                                   self.setRepository)
        self.connect(self.ui.actionFavourites,
                     QtCore.SIGNAL('triggered()'),
                     self.favourites)
        self.connect(self.ui.actionPreferences,
                     QtCore.SIGNAL('triggered()'),
                     self.prefs)
        self.connect(self.ui.commitToolButton,
                     QtCore.SIGNAL('clicked()'),
                     self.commit)
        self.connect(self.ui.historyToolButton,
                     QtCore.SIGNAL('clicked()'),
                     self.history)
        self.connect(self.ui.serverButton,
                     QtCore.SIGNAL('clicked()'),
                     self.serverToggle)
        self.connect(self.ui.actionQuit,
                     QtCore.SIGNAL('triggered()'),
                     self.close)
        self.connect(self.qapp,
                     QtCore.SIGNAL('lastWindowClosed()'),
                     self.cleanup)
        self.connect(self,
                     QtCore.SIGNAL('reallyquit()'),
                     self.qapp,
                     QtCore.SLOT('quit()'))
        self.connect(self.ui.goButton,
                     QtCore.SIGNAL('clicked()'),
                     self.setRepository)
        self.connect(self.ui.bundleButton,
                     QtCore.SIGNAL('clicked()'),
                     self.bundle)
        self.connect(self.ui.actionApply_Bundle,
                     QtCore.SIGNAL('triggered()'),
                     self.apply_bundle)
        self.connect(self.ui.actionApply_Patch,
                     QtCore.SIGNAL('triggered()'),
                     self.apply_patch)
        self.connect(self.ui.actionExport_Archive,
                     QtCore.SIGNAL('triggered()'),
                     self.export_archive)
        self.connect(self.ui.executeButton,
                     QtCore.SIGNAL('clicked()'),
                     self.execute_cmd)
        self.connect(self.ui.commandEdit,
                     QtCore.SIGNAL('returnPressed()'),
                     self.ui.executeButton,
                     QtCore.SLOT('click()'))
        self.connect(self.ui.actionTag_Current,
                     QtCore.SIGNAL('triggered()'),
                     self.tag_current)
        self.connect(self.ui.actionInit_Repo,
                     QtCore.SIGNAL('triggered()'),
                     self.init_repo)
        self.connect(self.ui.actionUpdate_Working_Tree,
                     QtCore.SIGNAL('triggered()'),
                     self.update)
        self.connect(self.ui.actionSend_Bundle,
                     QtCore.SIGNAL('triggered()'),
                     self.emailBundle)
        self.connect(self.ui.actionAnnotate_File,
                     QtCore.SIGNAL('triggered()'),
                     self.annotateFile)
        self.connect(self.ui.actionOptimize,
                     QtCore.SIGNAL('triggered()'),
                     self.optimize)

        os.environ['SSH_ASKPASS'] = '/usr/bin/ssh-askpass'
        sys.stdin.close()

        self.settings = QtCore.QSettings('WibbleSort', 'Push Me Pull You')
        self.serverRunning = False
        
        self.ui.currentRepo.setText(QtCore.QString(
                os.path.realpath(os.getcwd())))
        self.ui.currentRepo.setFocus(QtCore.Qt.OtherFocusReason)

        self.scm = None
        self.setRepository()
    
    def probeScm(self, name=None):
        self.serverRunning = False
        if name == None:
            self.scm = scm.probe()
        else:
            self.scm = scm.get_backend(name)

        print "scm = %s" % self.scm.name

        # If we succeed in finding an SCM then we should modify the
        # GUI to fit the available options

#        self.ui.actionClone.setEnabled(self.scm.features[scm.SCM_CLONE])
        self.ui.pushOut.setEnabled(self.scm.features[scm.SCM_PUSH])
        self.ui.pullIn.setEnabled(self.scm.features[scm.SCM_PULL])
        self.ui.bundleButton.setEnabled(self.scm.features[scm.SCM_CREATE_BUNDLE])
        self.ui.actionApply_Bundle.setEnabled(self.scm.features[scm.SCM_IMPORT_BUNDLE])
        # we don't yet have a ui option for SCM_EXPORT_PATCH
        self.ui.actionApply_Patch.setEnabled(self.scm.features[scm.SCM_IMPORT_PATCH])
        self.ui.serverButton.setEnabled(self.scm.features[scm.SCM_SERVER])
        self.ui.actionExport_Archive.setEnabled(self.scm.features[scm.SCM_ARCHIVE])
        self.ui.executeButton.setEnabled(self.scm.features[scm.SCM_EXECUTE])
        self.ui.actionTag_Current.setEnabled(self.scm.features[scm.SCM_TAG])
        self.ui.actionSend_Bundle.setEnabled(self.scm.features[scm.SCM_CREATE_BUNDLE])
        self.ui.actionUpdate_Working_Tree.setEnabled(self.scm.features[scm.SCM_UPDATE])
        self.ui.actionAnnotate_File.setEnabled(self.scm.features[scm.SCM_ANNOTATE])
        self.ui.actionOptimize.setEnabled(self.scm.features[scm.SCM_OPTIMIZE])
        
        self.ui.menuAlternative_Types.clear()
        self.ui.menuAlternative_Types.setEnabled(self.scm.features[scm.SCM_ALT_TYPES])
        if self.scm.features[scm.SCM_ALT_TYPES]:
            for t in self.scm.alt_types:
                action = self.ui.menuAlternative_Types.addAction(t)
                self.connect(action,
                             QtCore.SIGNAL('triggered()'),
                             self.switchScm)

    def switchScm(self):
        action = self.sender()
        self.setRepository(type=action.text())

    def callScm(self, fname, *args):
        return self.callOtherScm(self.scm, fname, *args)

    def callOtherScm(self, my_scm, fname, *args):
        self.setCursor(QtCore.Qt.WaitCursor)
        fn = getattr(my_scm, fname)
        ret = fn(*args)
        self.setCursor(QtCore.Qt.ArrowCursor)
        return ret

    def runExternal(self, cmdline):
        self.setCursor(QtCore.Qt.WaitCursor)
        os.system(cmdline)
        self.setCursor(QtCore.Qt.ArrowCursor)

    def optimize(self):
        self.callScm('optimize')

    def annotateFile(self):
        file = QtGui.QFileDialog.getOpenFileName(self)
        if file == '':
            return

        annotated_lines = self.callScm('annotate', file)
        
        ann = Annotate(annotated_lines)
        ann.exec_()

    def emailBundle(self):
        em = Email()
        if em.exec_() == 0:
            return

        bundle_dir = tempfile.mkdtemp()
        
        bundle_path = bundle_dir + '/' + em.bundleName()
        print 'Creating bundle at %s' % bundle_path
        self.callScm('create_bundle', bundle_path, self.destPath())

        mailprog = str( self.settings.value('preferences/email',
                                            QtCore.QVariant('')).toString() )

        mail_args = { 'to' : str( em.to() ),
                      'subject' : str( em.subject() ),
                      'message' : str( em.message() ),
                      'attach' : bundle_path }
        self.runExternal(mailprog % mail_args)

        os.unlink(bundle_path)
        os.rmdir(bundle_dir)

    def update(self):
        up = Update(self.scm)
        if up.exec_() == 0:
            return

        type = up.type()
        if type == scm.UPDATE_TO_TIP:
            self.callScm('update_to_tip')
        elif type == scm.UPDATE_TO_REV:
            self.callScm('update_to_rev', up.rev())
        elif type == scm.UPDATE_TO_TIME:
            self.callScm('update_to_time', up.time())

    def init_repo(self):
        init = InitRepo(scm.backends())
        if init.exec_() == 0:
            return

        scm_tmp = scm.get_backend(init.scmName())
        self.callOtherScm(scm_tmp, 'init', init.repo())

        self.ui.currentRepo.setText(init.repo())
        self.setRepository()

    def tag_current(self):
        tag = Tag()
        if tag.exec_() == 0:
            return

        self.callScm('tag', tag.name())

        self.updateDisplays(updateIncoming = False)

    def execute_cmd(self):
        cmd = str(self.ui.commandEdit.displayText())

        cmd_list = cmd.split()

        if len(cmd_list) == 0:
            print 'No command to execute'
            return

        if cmd_list[0] == self.scm.name:
            if len(cmd_list) > 1:
                self.callScm('execute', cmd_list[1], cmd_list[2:])
                self.updateDisplays() # In case command changed things!
            else:
                print 'Invalid SCM command'
        else:
            print 'Unrecognised PMPU command'

        self.ui.commandEdit.setText('')
        
    def export_archive(self):
        ea = Archive(self.scm)
        if ea.exec_() == 0:
            return
        
        file = str( ea.archiveBox.archiveFile.displayText() )
        type = str( ea.archiveBox.archiveTypes.currentText() )

        self.callScm('archive', file, type)

    def apply_patch(self):
        patch_path = QtGui.QFileDialog.getOpenFileName(self)
        # test for cancel
        
        self.callScm('import_patch', patch_path)
        self.updateDisplays() # changesets may have been imported

    def apply_bundle(self):
        bundle_path = QtGui.QFileDialog.getOpenFileName(self)
        # test for cancel
        
        self.callScm('import_bundle', bundle_path)
        self.updateDisplays()

    def bundle(self):
        bundle_path = QtGui.QFileDialog.getSaveFileName(self)
        # test for cancel
        self.callScm('create_bundle', bundle_path, self.destPath())

    def gui_elements_reset(self):
        self.ui.serverButton.setText('Start &Server')

    def cleanup(self):
        if self.scm:
            self.callScm('destroy')
        self.emit(QtCore.SIGNAL('reallyquit()'))

    def serverToggle(self):
        if self.serverRunning:
            self.callScm('stop_server')
            self.ui.serverButton.setText('Start &Server')
            self.serverRunning = False
        else:
            self.callScm('start_server')
            self.ui.serverButton.setText('Stop &Server')
            self.serverRunning = True


    def history(self):
        histview = self.settings.value('preferences/%s/histview' % self.scm.name,
                                       QtCore.QVariant('')).toString()
        self.runExternal(str(histview))

    def commit(self):
        ct = self.settings.value('preferences/%s/ctpath' % self.scm.name,
                                 QtCore.QVariant('')).toString()
        self.runExternal(str(ct))
        self.updateDisplays(updateIncoming=False)

    def prefs(self):
        p = Prefs(self.scm, self.settings)
        if p.exec_() == 0:
            return

        email = p.prefsBox.emailProg.displayText()
        ct = p.prefsBox.commitToolPath.displayText()
        histview = p.prefsBox.histViewPath.displayText()

        self.settings.setValue('preferences/email', QtCore.QVariant(email))
        self.settings.setValue('preferences/%s/ctpath' % self.scm.name,
                               QtCore.QVariant(ct))
        self.settings.setValue('preferences/%s/histview' % self.scm.name,
                               QtCore.QVariant(histview))

    def favourites(self):
        f = Favourites(self.settings)
        f.exec_()

    def showAbout(self):
        a = About()
        a.exec_()

    def destPath(self):
        return self.ui.paths.currentText()

    def clone(self):
        c = Clone(self.settings, scm.backends())
        if c.exec_() == 0:
            return
        newRepo = c.cloneDialog.destPath.displayText()
        
        scm_tmp = scm.get_backend(c.scmName())

        self.callOtherScm(scm_tmp, 'clone', c.cloneDialog.sourcePath.displayText(), newRepo)
        
        self.ui.currentRepo.setText(newRepo)
        self.setRepository()

    def openPath(self):
        path = QtGui.QFileDialog.getExistingDirectory(self)

        if path != '':
            self.ui.currentRepo.setText(path)
            self.setRepository()

    def updateDisplays(self, idx=0, updateOutgoing=True,
                       updateIncoming=True):
        path = str( self.ui.paths.currentText() )
        
        print "Changed path to %s!" % path
        print "CWD is %s" % os.curdir

        if path == '':
            self.ui.incomingBrowser.setText('No foreign repository selected.')
            self.ui.outgoingBrowser.setText('No foreign repository selected.')
            return

        # Update incoming and outgoing.
        if updateIncoming:
            if self.scm.features[scm.SCM_INCOMING]:
                in_csets = self.callScm('incoming', path)
            else:
                in_csets = QtCore.QString('Incoming changesets feature is unavailable.')
            self.ui.incomingBrowser.setText(in_csets)
            
        if updateOutgoing:
            if self.scm.features[scm.SCM_OUTGOING]:
                out_csets = self.callScm('outgoing', path)
            else:
                out_csets = QtCore.QString('Outgoing changesets feature is unavailable.')
            self.ui.outgoingBrowser.setText(out_csets)

    def setRepository(self, type=None):
        path = self.ui.currentRepo.displayText()
        print "Chose repository %s" % path

        if self.scm:
            self.callScm('destroy')

        self.gui_elements_reset()
        
        # Change to chosen repository.
        try:
            os.chdir(path)
        except OSError, e:
            print 'Error setting repository:'
            print e
            type = 'no_scm'
    
        self.probeScm(name = type)

        self.ui.currentRepo.setText(path)
        self.ui.paths.clear()
        self.ui.paths.clearEditText()
        self.ui.incomingBrowser.clear()
        self.ui.outgoingBrowser.clear()

        path_list = []
        self.updateDisplays()

        if self.scm.features[scm.SCM_PATHS]:
            path_list = self.callScm('paths')

        for name, value in path_list:
            self.ui.paths.addItem(value)

        

    def push(self):
        self.callScm('push', self.ui.paths.currentText())
        self.updateDisplays(updateIncoming=False)

    def pull(self):
        self.callScm('pull', self.ui.paths.currentText())
        self.updateDisplays()

