SCM_INIT = 'init'
SCM_CLONE = 'clone'
SCM_PATHS = 'paths'
SCM_PUSH = 'push'
SCM_PULL = 'pull'
SCM_INCOMING = 'incoming'
SCM_OUTGOING = 'outgoing'
SCM_CREATE_BUNDLE = 'create bundle'
SCM_IMPORT_BUNDLE = 'import bundle'
SCM_EXPORT_PATCH = 'export patch'
SCM_IMPORT_PATCH = 'import patch'
SCM_SERVER = 'server'
SCM_ARCHIVE = 'archive'
SCM_EXECUTE = 'execute'
SCM_TAG = 'tag'
SCM_UPDATE = 'update'
SCM_ANNOTATE = 'annotate'
SCM_OPTIMIZE = 'optimize'
SCM_ALT_TYPES = 'alternative types'

ARCHIVE_FILES = 'plain files'
ARCHIVE_TAR = 'tar file'
ARCHIVE_TAR_BZ2 = 'tar.bz2 file'
ARCHIVE_TAR_GZ = 'tar.gz file'
ARCHIVE_ZIP_UNCOMPRESSED = 'uncompressed zip file'
ARCHIVE_ZIP_COMPRESSED = 'deflated zip file'

UPDATE_TO_TIP = 'update to tip'
UPDATE_TO_REV = 'update to rev'
UPDATE_TO_TIME = 'update to time'

scm_backends = []

def register_backend(backend):
    scm_backends.append(backend)

def probe():
    for s in scm_backends:
        p = s.probe()
        if p:
            return p
    
    return scm()

def backends():
    names = []
    return scm_backends

def get_backend(name):
    for s in scm_backends:
        if s.name == name:
            return s()

class annotatedLine:
    """Represents a single line of source code, annotated with SCM data."""
    def __init__(self, user, revid, revno, date, file, text):
        self.user = user
        self.revid = revid
        self.revno = revno
        self.date = date
        self.file = file
        self.text = text

    def __getitem__(self, item):
        if item == 'user':
            return self.user
        elif item == 'revid':
            return self.revid
        elif item == 'revno':
            return self.revno
        elif item == 'date':
            return self.date
        elif item == 'file':
            return self.file
        elif item == 'text':
            return self.text

    def toText(self):
        return '%s %s %s %s %s %s' % (self.user, self.revid, self.revno,
                                      self.date, self.file, self.text)

class scmException(Exception):
    pass

class scm:

    name = "no_scm"


    features = { SCM_INIT : False,
                 SCM_CLONE : False,
                 SCM_PATHS : False,
                 SCM_PUSH : False,
                 SCM_PULL : False,
                 SCM_INCOMING : False,
                 SCM_OUTGOING : False,
                 SCM_CREATE_BUNDLE : False,
                 SCM_IMPORT_BUNDLE : False,
                 SCM_EXPORT_PATCH : False,
                 SCM_IMPORT_PATCH : False,
                 SCM_SERVER : False,
                 SCM_ARCHIVE : False,
                 SCM_EXECUTE : False,
                 SCM_TAG : False,
                 SCM_UPDATE : False,
                 SCM_ANNOTATE : False,
                 SCM_OPTIMIZE : False,
                 SCM_ALT_TYPES : False, }

    archive_formats = { ARCHIVE_FILES : False,
                        ARCHIVE_TAR : False,
                        ARCHIVE_TAR_BZ2 : False,
                        ARCHIVE_TAR_GZ : False,
                        ARCHIVE_ZIP_UNCOMPRESSED : False,
                        ARCHIVE_ZIP_COMPRESSED : False, }
    
    update_modes = { UPDATE_TO_TIP : False,
                     UPDATE_TO_REV : False,
                     UPDATE_TO_TIME : False, }

    def __init__(self):
        pass

    @staticmethod
    def probe():
        return None
    
    def clone(self):
        raise scmException('Clone not supported')

    def paths(self):
        raise scmException('Paths not supported')

    def push(self):
        raise scmExeption('Push not supported')

    def pull(self):
        raise scmException('Pull not supported')

    def incmoing(self):
        raise scmException('Incoming not supported')

    def outgoing(self):
        raise scmException('Outgoing not supported')

    def create_bundle(self):
        raise scmException('Create bundle not supported')

    def import_bundle(self):
        raise scmException('Import bundle not supported')

    def export_patch(self):
        raise scmException('Export patch not supported')

    def import_patch(self):
        raise scmException('Import patch not supported')

    def start_server(self):
        raise scmException('Server not supported')

    def stop_server(self):
        raise scmException('Server not supported')

    def archive(self):
        raise scmException('Archive not supported')

    def execute(self, cmd_list):
        raise scmException('Execute not supported')

    def destroy(self):
        pass

register_backend(scm)

import scm_hg, scm_hgforest, scm_bzr, scm_git, scm_darcs
