import os, signal, re

from scm import *

class bzr(scm):

    name = 'bzr'

    features = { SCM_INIT : True,
                 SCM_CLONE : True,
                 SCM_PATHS : True,
                 SCM_PUSH : True,
                 SCM_PULL : True,
                 SCM_INCOMING : True,
                 SCM_OUTGOING : True,
                 SCM_CREATE_BUNDLE : True,
                 SCM_IMPORT_BUNDLE : True,
                 SCM_IMPORT_PATCH : False,
                 SCM_EXPORT_PATCH : False,
                 SCM_SERVER : True,
                 SCM_ARCHIVE: True,
                 SCM_EXECUTE: True,
                 SCM_TAG : True,
                 SCM_UPDATE : True,
                 SCM_ANNOTATE : True,
                 SCM_OPTIMIZE : True,
                 SCM_ALT_TYPES : False }
    
    archive_formats = { ARCHIVE_FILES : True,
                        ARCHIVE_TAR : True,
                        ARCHIVE_TAR_BZ2 : True,
                        ARCHIVE_TAR_GZ : True,
                        ARCHIVE_ZIP_COMPRESSED : True }
    
    update_modes = { UPDATE_TO_TIP : True,
                     UPDATE_TO_REV : True,
                     UPDATE_TO_TIME : False, }
    
    time_format = 'yyyy-MM-dd hh:mm:ss'

    def __init__(self):
        scm.__init__(self)

        self.serverpid = 0

    @staticmethod
    def probe():
        if os.path.exists('.bzr'):
            print 'Detected a bzr repository'
            return bzr()
        else:
            return None

    def execute(self, cmd, args):
        arg_str = ''
        for a in args:
            arg_str += ' ' + a

        output = os.popen('bzr ' + cmd + str(arg_str)).read()
        return output

    def optimize(self):
        self.execute('pack', [])

    def annotate(self, file):
        annotated_lines = []

        text = self.execute('annotate', ['--long', '--all', file])
        
        annotate_re = re.compile(r"(?P<revno>\S*)\s*(?P<user>\S*)\s*(?P<date>\S*) \| (?P<text>.*)")
        
        for line in text.split('\n'):
            m = annotate_re.match(line)
            if m == None:
                print 'Failed to match!'
                print 'line was: %s' % line
                continue
            user = m.group('user')
            revno = m.group('revno')
            date = m.group('date')
            text = m.group('text')

            ann_line = annotatedLine(user, None, revno, date, None, text)
            annotated_lines.append(ann_line)

        return annotated_lines

    def update_to_tip(self):
        self.execute('revert', [])

    def update_to_rev(self, rev):
        self.execute('revert', ['-r', rev])

    def update_to_time(self, time):
        self.execute('revert', ['-r', '"before:date:%s"' % time])

    def init(self, path):
        self.execute('init', [path])

    def clone(self, source_path, new_path):
        self.execute('branch', [source_path, new_path])

    def paths(self):
        path_list = []

        if os.path.exists('.bzr/branch/branch.conf'):
            branchconf = open('.bzr/branch/branch.conf')

            for line in branchconf.readlines():
                m = re.match(r"(\S*)\s*=\s*(\S*)\s*", line)
                if m == None:
                    continue

                name = m.group(1)
                value = m.group(2)

                if name == None or value == None:
                    continue

                if name == 'parent_location' or name == 'push_location':
                    print "Got path %s = %s" % (name, value)
                    path_list.append((name, value))

            branchconf.close()
                    
        if os.path.exists('.bzr/branch/parent'):
            parent_file = open('.bzr/branch/parent')
            parent = parent_file.readline().strip()
            path_list.append(('parent', parent))
            parent_file.close()

        return path_list

    def push(self, path):
        print "bzr push"
        self.execute('push', [path])

    def pull(self, path):
        print "bzr merge"
        self.execute('merge', [path])

    def incoming(self, path):
        return self.execute('missing', ['--theirs-only', path])

    def outgoing(self, path):
        return self.execute('missing', ['--mine-only', path])

    def create_bundle(self, bundle_path, destPath):
        self.execute('send', [destPath, '-o', bundle_path])

    def import_bundle(self, bundle_path):
        self.execute('pull', [bundle_path])

    def archive(self, path, type):
        type_arg = ''
        if type == ARCHIVE_FILES:
            type_arg = 'dir'
        elif type == ARCHIVE_TAR:
            type_arg = 'tar'
        elif type == ARCHIVE_TAR_BZ2:
            type_arg = 'tbz2'
        elif type == ARCHIVE_TAR_GZ:
            type_arg = 'tgz'
        elif type == ARCHIVE_ZIP:
            type_arg = 'zip'

        return self.execute('export', ['--format=%s' % type_arg, path])

    def start_server(self):
        self.serverpid = os.fork()
        if not self.serverpid:
            os.execlp('bzr', 'bzr', 'serve')

    def stop_server(self):
        if self.serverpid:
            os.kill(self.serverpid, signal.SIGTERM)
            self.serverpid = 0
            os.wait()
            
    def tag(self, name):
        self.execute('tag', [name])

    def destroy(self):
        self.stop_server()

register_backend(bzr)

