import os, signal
import curses.ascii

#from ConfigParser import SafeConfigParser

from scm import *

class darcs(scm):

    name = 'darcs'

    features = { SCM_INIT : True,
                 SCM_CLONE : True,
                 SCM_PATHS : True,
                 SCM_PUSH : True,
                 SCM_PULL : True,
                 SCM_INCOMING : True,
                 SCM_OUTGOING : True,
                 SCM_CREATE_BUNDLE : True,
                 SCM_IMPORT_BUNDLE : True,
                 SCM_EXPORT_PATCH : False,
                 SCM_IMPORT_PATCH : False,
                 SCM_SERVER : False,
                 SCM_ARCHIVE : True,
                 SCM_EXECUTE : True,
                 SCM_TAG : True,
                 SCM_UPDATE : False,
                 SCM_ANNOTATE : False,
                 SCM_OPTIMIZE : True,
                 SCM_ALT_TYPES : False, }
    
    archive_formats = { ARCHIVE_FILES : False,
                        ARCHIVE_TAR : False,
                        ARCHIVE_TAR_BZ2 : False,
                        ARCHIVE_TAR_GZ : True,
                        ARCHIVE_ZIP_UNCOMPRESSED : False,
                        ARCHIVE_ZIP_COMPRESSED : False, }
    
    def __init__(self):
        scm.__init__(self)

        self.serverpid = 0

    @staticmethod
    def probe():
        if os.path.exists('_darcs'):
            print 'Detected a Darcs repository'
            return darcs()
        else:
            return None

    def execute(self, cmd, args):
        arg_str = ''
        for a in args:
            arg_str += ' ' + a

        output = os.popen('darcs ' + cmd + str(arg_str)).read()
        return output

    def optimize(self):
        self.execute('optimize', [])

    def init(self, path):
        if os.path.exists(path) == False:
            os.makedirs(path)
        
        self.execute('init', ['--repodir=%s' % path])

    def clone(self, source_path, new_path):
        self.execute('get', [source_path, new_path])

    def paths(self):
        path_list = []
        if os.path.exists('_darcs/prefs/repos'):
            repo_file = open('_darcs/prefs/repos')
            for line in repo_file:
                path_list.append(('', line.strip()))

        return path_list

    def push(self, path):
        print "darcs push"
        self.execute('push', ['--all', path])

    def pull(self, path):
        print "darcs pull"
        self.execute('pull', ['--all', path])

    # This filters out some control characters in the console output
    # that I'm not entirely clear on the purpose of.  They were
    # messing up the output, anyhow, so I filter them.  Maybe some
    # Darcs people can explain what they are?
    def filter_pushpull_chars(self, data):
        ret = ''
        lines = data.split('\n')
        for l in lines:
            idx = l.find('arg' + chr(curses.ascii.DEL) + 'darcdarc')
            if idx == -1:
                ret += (l + '\n')
                continue
            ret += (l[:idx] + '\n')
        return ret

    def incoming(self, path):
        in_csets = self.execute('pull', ['--dry-run', '--all', path])
        in_csets = self.filter_pushpull_chars(in_csets)
        return in_csets

    def outgoing(self, path):
        out_csets = self.execute('push', ['--dry-run', '--all', path])
        out_csets = self.filter_pushpull_chars(out_csets)
        return out_csets
    
    def create_bundle(self, bundle_path, destPath):
        self.execute('send', ['--all', '-o', bundle_path, destPath])

    def import_bundle(self, bundle_path):
        self.execute('apply', ['--all', bundle_path])

    def archive(self, path, type):
        dir, name = os.path.split(path) # Remove base path, darcs creates in working dir by default.
        if dir != '' and dir != os.getcwd():
            print 'ERROR: can only export from darcs to archives in repository root'
        # Currently the type can only be ARCHIVE_TAR_GZ
        if name.endswith('.tar.gz'):
            name = name[:-len('.tar.gz')] # Remove extension; darcs adds that.
        elif name.endswith('.tgz'):
            print 'ERROR: only the .tar.gz extension is supported by darcs'
            return
            name = name[:-len('.tgz')] # BUG: will save as .tar.gz even if .tgz specified
        else:
            print 'ERROR: only the .tar.gz extension is supported by darcs'
            return

        self.execute('dist', [ '-d', name])

    def tag(self, name):
        self.execute('tag', [name])
        
    def destroy(self):
        pass

register_backend(darcs)

