import os, signal, re
from ConfigParser import SafeConfigParser

from scm import *

class hg(scm):

    name = 'hg'

    features = { SCM_INIT : True,
                 SCM_CLONE : True,
                 SCM_PATHS : True,
                 SCM_PUSH : True,
                 SCM_PULL : True,
                 SCM_INCOMING : True,
                 SCM_OUTGOING : True,
                 SCM_CREATE_BUNDLE : True,
                 SCM_IMPORT_BUNDLE : True,
                 SCM_EXPORT_PATCH : False,
                 SCM_IMPORT_PATCH : True,
                 SCM_SERVER : True,
                 SCM_ARCHIVE : True,
                 SCM_EXECUTE : True,
                 SCM_TAG : True,
                 SCM_UPDATE : True,
                 SCM_ANNOTATE : True,
                 SCM_OPTIMIZE : False,
                 SCM_ALT_TYPES : True, }
    
    archive_formats = { ARCHIVE_FILES : True,
                        ARCHIVE_TAR : True,
                        ARCHIVE_TAR_BZ2 : True,
                        ARCHIVE_TAR_GZ : True,
                        ARCHIVE_ZIP_UNCOMPRESSED : True,
                        ARCHIVE_ZIP_COMPRESSED : True, }
    
    update_modes = { UPDATE_TO_TIP : True,
                     UPDATE_TO_REV : True,
                     UPDATE_TO_TIME : False, } # disabled due to hg bug
    
    alt_types = ['hg forest']
    
    time_format = 'yyyy-MM-dd hh:mm:ss'

    def __init__(self):
        scm.__init__(self)

        self.serverpid = 0

    @staticmethod
    def probe():
        if os.path.exists('.hg'):
            print 'Detected a Mercurial repository'
            return hg()
        else:
            return None

    def execute(self, cmd, args):
        arg_str = ''
        for a in args:
            arg_str += ' ' + a

        output = os.popen('hg ' + cmd + str(arg_str)).read()
        return output

    def annotate(self, file):
        annotated_lines = []

        text = self.execute('annotate', ['-f', # Follow copies and renames.
                                         '-u', # List author.
                                         '-d', # List date.
                                         '-n', # List revision number.
                                         '-c', # List changeset.
                                         file
                                         ])

        annotate_re = re.compile(r"(?P<user>\S*)\s*(?P<revno>\d*)\s*(?P<cset>\S*)\s*"
                                 + r"(?P<date>... ... .. ........ .... .....)\s*"
                                 + r"(?P<file>\S*): (?P<text>.*$)")

        for line in text.split('\n'):
            m = annotate_re.match(line)
            if m == None:
                print 'Failed to match!'
                print 'line was: %s' % line
                continue
            user = m.group('user')
            revno = m.group('revno')
            cset = m.group('cset')
            date = m.group('date')
            file = m.group('file')
            text = m.group('text')

            ann_line = annotatedLine(user, cset, revno, date, file, text)
            annotated_lines.append(ann_line)

        return annotated_lines

    def update_to_tip(self):
        self.execute('update', ['tip'])

    def update_to_rev(self, rev):
        self.execute('update', [rev])

    def update_to_time(self, time):
        self.execute('update', ['-d', '"%s"' % time])

    def init(self, path):
        self.execute('init', [path])

    def clone(self, source_path, new_path):
        self.execute('clone', [source_path, new_path])

    def paths(self):
        path_list = []

        cp = SafeConfigParser()
        cp.read('.hg/hgrc') # failure case!  other hgrcs influence?

        if cp.has_section('paths'):
            for name, value in cp.items('paths'):
                print "Got path %s = %s" % (name, value)
                path_list.append((name, value))

        return path_list

    def push(self, path):
        print "hg push"
        self.execute('push', [path])

    def pull(self, path):
        print "hg pull"
        self.execute('pull', [path])
        
    def incoming(self, path):
        return self.execute('incoming', [path])

    def outgoing(self, path):
        return self.execute('outgoing', [path])

    def create_bundle(self, bundle_path, destPath):
        self.execute('bundle', [bundle_path, destPath])

    def import_bundle(self, bundle_path):
        self.execute('unbundle', [bundle_path])

    def export_patch(self):
        raise scmException('Export patch not supported')

    def import_patch(self, patch_path):
        self.execute('patch', [patch_path])

    def start_server(self):
        self.serverpid = os.fork()
        if not self.serverpid:
            os.execlp('hg', 'hg', 'serve')

    def stop_server(self):
        if self.serverpid:
            os.kill(self.serverpid, signal.SIGTERM)
            self.serverpid = 0
            os.wait()

    def archive(self, path, type):
        type_arg = ''
        if type == ARCHIVE_FILES:
            type_arg = 'files'
        elif type == ARCHIVE_TAR:
            type_arg = 'tar'
        elif type == ARCHIVE_TAR_BZ2:
            type_arg = 'tbz2'
        elif type == ARCHIVE_TAR_GZ:
            type_arg = 'tgz'
        elif type == ARCHIVE_ZIP_UNCOMPRESSED:
            type_arg = 'uzip'
        elif type == ARCHIVE_ZIP_COMPRESSED:
            type_arg = 'zip'

        self.execute('archive', ['-t', type_arg, path])

    def tag(self, name):
        self.execute('tag', [name])

    def destroy(self):
        self.stop_server()


register_backend(hg)

