import os, signal, re
from ConfigParser import SafeConfigParser

from scm import *

class hgforest(scm):

    name = 'hg forest'

    features = { SCM_INIT : False,
                 SCM_CLONE : True,
                 SCM_PATHS : True,
                 SCM_PUSH : True,
                 SCM_PULL : True,
                 SCM_INCOMING : True,
                 SCM_OUTGOING : True,
                 SCM_CREATE_BUNDLE : False,
                 SCM_IMPORT_BUNDLE : False,
                 SCM_EXPORT_PATCH : False,
                 SCM_IMPORT_PATCH : False,
                 SCM_SERVER : False, # Disabled because fclone doesn't work for me?
                 SCM_ARCHIVE : False,
                 SCM_EXECUTE : True,
                 SCM_TAG : False,
                 SCM_UPDATE : False,
                 SCM_ANNOTATE : False,
                 SCM_OPTIMIZE : False,
                 SCM_ALT_TYPES : True, }
    
    archive_formats = { ARCHIVE_FILES : True,
                        ARCHIVE_TAR : True,
                        ARCHIVE_TAR_BZ2 : True,
                        ARCHIVE_TAR_GZ : True,
                        ARCHIVE_ZIP_UNCOMPRESSED : True,
                        ARCHIVE_ZIP_COMPRESSED : True, }
    
    update_modes = { UPDATE_TO_TIP : True,
                     UPDATE_TO_REV : True,
                     UPDATE_TO_TIME : False, } # disabled due to hg bug
    
    alt_types = ['hg']
    
    time_format = 'yyyy-MM-dd hh:mm:ss'

    def __init__(self):
        scm.__init__(self)

        self.serverpid = 0

    @staticmethod
    def probe():
        return None

    def execute(self, cmd, args=[]):
        arg_str = ''
        for a in args:
            arg_str += ' ' + a

        output = os.popen('hg ' + cmd + str(arg_str)).read()
        return output

    def trees(self):
        output = self.execute('ftrees')
        ret = output.split('\n')
        print ret
        return ret

    def update_to_tip(self):
        self.execute('fupdate', ['tip'])

    def update_to_rev(self, rev):
        self.execute('fupdate', [rev])

    def init(self, path):
        self.execute('init', [path])

    def clone(self, source_path, new_path):
        self.execute('fclone', [source_path, new_path])

    def paths(self):
        path_list = []

        cp = SafeConfigParser()
        cp.read('.hg/hgrc') # failure case!  other hgrcs influence?

        if cp.has_section('paths'):
            for name, value in cp.items('paths'):
                print "Got path %s = %s" % (name, value)
                path_list.append((name, value))

        return path_list

    def push(self, path):
        print "hg fpush"
        self.execute('fpush', [path])

    def pull(self, path):
        print "hg fpull"
        self.execute('fpull', [path])
        
    def incoming(self, path):
        return self.execute('fincoming', [path])

    def outgoing(self, path):
        return self.execute('foutgoing', [path])

    def destroy(self):
        pass

register_backend(hgforest)
